from paver.easy import *
from paver.setuputils import setup

options = environment.options

NAME = 'blackbelt'
VERSION = '0.11.1'

requires = ['click', 'requests', 'PyGithub']

setup(
    name=NAME,
    version=VERSION,
    description='Project automation the Apiary way',
    long_description="""
    Black Belt: Project Automation The Apiary Way
    ==================================================

    Integrates useful GitHub workflow with Trello, Hipchat and Circle CI.

    Please visit `the Black Belt documentation <http://black-belt.readthedocs.org/en/latest/>`_ for more details.
    """,
    author='Lukas Linhart',
    author_email='lukas@apiary.io',
    url='http://github.com/apiaryio/black-belt',
    license='MIT',
    packages=[NAME, NAME + '.apis', NAME + '.commands'],
    install_requires=requires,
    # requires=requires,
    tests_require=['nose', 'virtualenv'],
    test_suite='nose.collector',
    zip_safe=False,
    # Yes, entry_points are nice and shiny and should be used in 21st century
    # Unfortunately, having some dependencies, the load_entry_point in the ge-
    # nerated console script blows up with pkg_resources.DistributionNotFound: requests
    # if it was installed with pip.
    # Pradoxically, solution is to easy_install -U all the dependencies.
    # Therefore, just fall back to the good, old, simple disutils script that works.
    scripts=['distutils_scripts/bb'],
    # entry_points={
    #     'console_scripts': [
    #         'bb = blackbelt.tasks:main'
    #     ]
    # },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Environment :: Console",
        "Topic :: Documentation",
        "Topic :: Utilities",
        "Topic :: Software Development :: Build Tools",
    ]
)

options.setup.package_data = paver.setuputils.find_package_data("blackbelt", package="blackbelt",
                                                only_in_packages=False)


@task
@consume_args
def bump(args):
    version = map(int, VERSION.split('.')[0:3])

    if len(args) > 0 and args[0] == 'major':
        version[1] += 1
        version[2] = 0
    else:
        version[2] += 1

    version = map(str, version)

    module_content = "VERSION='%s'\n" % '.'.join(version)

    # bump version in blackbelt
    with open(path('blackbelt/version.py'), 'w') as f:
        f.write(module_content)

    # bump version in sphinx
    conf = []
    with open(path('docs/source/conf.py'), 'r') as f:
        for line in f.readlines():
            if line.startswith('version = '):
                line = "version = '%s'\n" % '.'.join(version[0:2])
            elif line.startswith('release = '):
                line = "release = '%s'\n" % '.'.join(version[0:3])

            conf.append(line)

    with open(path('docs/source/conf.py'), 'w') as f:
        f.writelines(conf)

    # bump version in this pavement file itself
    # we cannot just import it because it implies blackbelt already been
    # installed/in path, which may cause some interesting problems
    # solvable by mangling with sys.path, but this just feels...better
    conf = []
    with open(path('./pavement.py'), 'r') as f:
        for line in f.readlines():
            if line.startswith('VERSION = '):
                line = "VERSION = '%s'\n" % '.'.join(version)

            conf.append(line)

    with open(path('./pavement.py'), 'w') as f:
        f.writelines(conf)

    sh("git commit pavement.py blackbelt/version.py docs/source/conf.py -m 'Version bump to %s'" % '.'.join(version))


@task
def release():
    sh("git tag -s '%(name)s-%(version)s' -m 'Version bump to %(version)s'" % {'name': NAME, 'version': VERSION})
    sh("git push --tags")
    sh("python setup.py register sdist upload")
    sh("python setup.py bdist_wheel upload")





from os.path import exists
from subprocess import check_call


def notify(title, message):
    # MacOS
    if exists('/usr/bin/osascript'):
        try:
            check_call([
                '/usr/bin/osascript', '-e',
                'display notification "{}" with title "{}"'.format(message, title)
            ])
        except Exception:
            print "[Can't notify user using osascript]"

    # libnotify (many Linux distros)
    elif exists('/usr/bin/notify-send'):
        try:
            check_call(['/usr/bin/notify-send', title, message])
        except Exception:
            print "[Can't notify user using libnotify]"



from subprocess import check_call

from blackbelt.handle_github import get_current_branch
from blackbelt.hipchat import post_message


def deploy_staging():
    branch_name = get_current_branch()

    post_message("@here Deploying branch %s to staging" % branch_name)

    check_call(['grunt', 'deploy', '--app=apiary-staging', '--force', "--branch=%s" % branch_name])



from nose.plugins.skip import SkipTest

import os

from blackbelt.config import config

from blackbelt.circle import Client

"""
Integration exploratory playground for CircleCI.
Don't really treat it as a test, but it may still be useful for you.
"""


class TestBuildInfoRetrieval(object):

    def setUp(self):
        try:
            self.client = Client()
        except ValueError:
            raise SkipTest()

        if os.environ.get('FORCE_DISCOVERY') != '1':
            raise SkipTest()

    def test_repo_owner(self):
        builds = self.client.get_builds('apiaryio', 'ivy')
        # print builds[0]
        assert len(builds) > 0




import json
from os.path import expanduser


try:
    configFile = expanduser('~/.blackbelt')
    with open(configFile) as f:
        config = json.loads(f.read())
except IOError:
    config = {
        'trello': {
            'access_token': None,
            'work_board_url': None,
            'work_board_id': None,
            'product_board_url': None,
            'product_board_id': None
        },
        'github': {
            'access_token': None
        },
        'circleci': {
            'access_token': None
        },
        'hipchat': {
            'access_token': None
        }
    }

# default config, overwrite/modularize
if not 'work_board_id' in config['trello'] or config['trello']['work_board_id']:
    config['trello']['work_board_id'] = '1KsoiV9e'

config['trello']['work_column_name'] = 'Doing'
config['trello']['pause_column_name'] = 'Ready'



import click

from blackbelt.deployment import (
    deploy_staging
)


@click.group(invoke_without_command=True, help="""
    Deploy current branch to staging
    """)
def cli():
    deploy_staging()



from nose.plugins.skip import SkipTest

import os

from blackbelt.apis.hipchat import HipChat

"""
Integration exploratory playground for CircleCI.
Don't really treat it as a test, but it may still be useful for you.
"""


class TestBuildInfoRetrieval(object):

    def setUp(self):
        try:
            self.client = HipChat()
        except ValueError:
            raise SkipTest()

        if os.environ.get('FORCE_DISCOVERY') != '1':
            raise SkipTest()

    def test_posting_message(self):
        self.client.post_message("Testing HipChat Messages in Black Belt Exploratory Test Suite")



import click

from blackbelt.handle_trello import (
    migrate_label as ml,
    schedule_list as sl,
    next_card as n,
    next_week as nw,
    open_current_working_ticket as cc,
    verify as v
)


@click.group(help='Handle Trello-related actions and integrations.')
def cli():
    pass


@cli.command(name='curcard')
def curcard():
    """ Open current doing card in browser """
    cc()


@cli.command(name='migrate-label')
@click.option('--label', default='', help='Label to migrate away')
@click.option('--board', help='Board to migrate from')
@click.option('--board-to', help='Board to migrate to')
@click.option('--column', help='Column to migrate from')
@click.option('--column-to', help='Column to migrate to')
def migrate_label(*args, **kwargs):
    ml(**kwargs)


@cli.command(name='schedule-list')
@click.option('--story-list', help='Name of the list that should be converted to cards')
@click.option('--owner', help='Whom to assign a created work card')
@click.option('--label', help='Assign a label to the newly-created card. For now, label must be a color name.')
@click.argument('story_card')
def schedule_list(**kwargs):
    """ Takes a TODO checklist on a given Story Card. Converts the items into the Work Overview Cards. """
    sl(**kwargs)


@cli.command()
def next(**kwargs):
    """ Takes the top card from To Do queue, moves it to doing and creates proper branch """
    n(**kwargs)


@cli.command(name='next-week')
def next_week():
    """ Creates new columns for this week """
    nw()


@cli.command()
@click.argument('story_card')
def verify(**kwargs):
    """ Verify given story card  """
    #TODO: Scan through all cards in the "Being worked on" column on story board
    v(**kwargs)



from datetime import datetime, timedelta
from time import sleep
import sys

import requests

from .config import config

MAX_WAIT_MINUTES = 120


class Client(object):

    def __init__(self, token=None):
        if not token:
            token = config['circleci']['access_token']

        if not token:
            raise ValueError("Can't do things with CircleCI without access token. Run bb init.")

        self.token = token

        self.default_headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }

        self.endpoint = 'https://circleci.com/api/v1'

    def get_url(self, url, **kwargs):
        kwargs['token'] = self.token

        url = self.endpoint + url + '?circle-token={token}'
        return url.format(**kwargs)

    def request(self, method, url, data=None):
        res = getattr(requests, method)(url, headers=self.default_headers)
        if not res.ok:
            raise ValueError("Request to URL %s failed" % url)

        return res.json()

    def get_builds(self, username, project):
        url = self.get_url(
            '/project/{username}/{project}',
            username=username,
            project=project
        )

        return self.request('get', url)

    def get_build(self, username, project, number):
        url = self.get_url(
            '/project/{username}/{project}/{number}',
            username=username,
            project=project,
            number=number
        )

        return self.request('get', url)


def wait_for_tests(sha, owner, name, retry_interval=30):
    client = Client()
    builds = client.get_builds(username=owner, project=name)
    build = None

    for b in builds:
        if b['vcs_revision'] == sha:
            build = b
            break

    if not build:
        raise ValueError("Cannot find build for our revision. TODO: Implement retry")

    start_time = datetime.now()

    while not build.get('stop_time'):
        if datetime.now() < start_time + timedelta(minutes=MAX_WAIT_MINUTES):
            sys.stdout.write('.')
            sys.stdout.flush()
            sleep(retry_interval)
        else:
            raise ValueError("Build haven't finished in time")

        build = client.get_build(
            username=owner,
            project=name,
            number=build['build_num']
        )

    # guards to prevent false positives
    assert build['build_time_millis'] > 0

    if build['status'] not in ['success', 'fixed']:
        raise ValueError("Build failed with status %(status)s" % build)

    return build



import os

import click

from blackbelt.configure import configure_blackbelt


@click.group(invoke_without_command=True, help="""Initialize application for usage. Invoke this command first.""")
def cli():
    configure_blackbelt()




from nose.tools import assert_equals, assert_raises
from mock import patch, MagicMock

import requests

from blackbelt.handle_github import (
    get_remote_repo_info,
    get_pr_info,
    verify_merge,
    get_pr_ticket_id
)


class TestGithubRepoParsing(object):

    github_repo = 'git@github.com:apiaryio/apiary-test.git'

    def setUp(self):
        self.parsed = get_remote_repo_info(self.github_repo)

    def test_repo_owner(self):
        assert_equals('apiaryio', self.parsed['owner'])

    def test_repo_name(self):
        assert_equals('apiary-test', self.parsed['name'])


class TestGithubPullRequestParsing(object):

    github_pr = 'https://github.com/apiaryio/apiary-test/pull/123'

    def setUp(self):
        self.parsed = get_pr_info(self.github_pr)

    def test_repo_owner(self):
        assert_equals('apiaryio', self.parsed['owner'])

    def test_repo_name(self):
        assert_equals('apiary-test', self.parsed['name'])

    def test_pr_number(self):
        assert_equals('123', self.parsed['number'])

    def test_files_link(self):
        parsed = get_pr_info(self.github_pr + '/files')
        assert_equals('123', parsed['number'])


    def test_trello_id_extracted(self):
        link = get_pr_ticket_id("""
        # This is edited and a long pull requests
        Hoever, I'd like to warn you...

        Pull request for [Naming fixes](https://trello.com/c/yx2SNE3J/1910-naming-fixes).
        """)

        assert_equals('yx2SNE3J', link)

    def test_error_on_bad_phrase(self):
        assert_raises(ValueError, lambda:get_pr_ticket_id("""
        Related to [naming fixes](https://trello.com/c/yx2SNE3J/1910-naming-fixes)
        """))



class TestMergeVerificationRetries(object):
    pr_info = {
        'owner': 'apiaryio',
        'name': 'apiary-test',
        'number': '666'
    }

    headers = {}

    @patch.object(requests, 'get')
    def test_timeout(self, get_method):
        get_method.return_value = fake_response = MagicMock()
        fake_response.status_code = 404

        assert_raises(ValueError, lambda: verify_merge(
            self.pr_info,
            self.headers,
            max_waiting_time=0.01,
            retry_time=0.001
        ))



from nose.tools import assert_equals, assert_raises

from blackbelt.handle_trello import infer_branch_name

class TestInferringBranch(object):

    def test_simple_url(self):
        assert_equals('bb-t-next', infer_branch_name('https://trello.com/c/7b4Z3V8o/1803-bb-t-next'))



import json
import re
import requests
from urllib import quote, quote_plus

from blackbelt.config import config
from blackbelt.errors import ConfigurationError

class Trello(object):
    """
    I represent a authenticated connection to Trello API.
    Dispatch all requests to it through my methods.

    My actions are named from the BlackBelt's POW; I don't aim to be a full,
    usable client.
    """

    API_KEY = "2e4bb3b8ec5fe2ff6c04bf659ee4553b"
    APP_NAME = 'black-belt'
    URL_PREFIX = "https://trello.com/1"

    def __init__(self, access_token=None):
        self._access_token = access_token
        if not self._access_token and config.get('trello') and config['trello'].get('access_token'):
            self._access_token = config['trello']['access_token']

    
    ### Infra
    def do_request(self, url, method='get', data=None):
        if not self._access_token:
            raise ConfigurationError("Trying to talk to Trello without having access token")

        url = self.URL_PREFIX + url
        response = getattr(requests, method)(
            url,
            params={
                'key': self.API_KEY,
                'token': self._access_token
            },
            data=data
        )
        # try:
        #     print response.text
        # except Exception:
        #     print 'Cannot print'
        response.raise_for_status()
        return json.loads(response.content)

    def get_card_id(self, card_url):
        # Trailing .* to accept longlings as well. Brittle, but that's how they work
        # See https://twitter.com/almadcz/status/537187876191350784
        match = re.match(r"^https://trello.com/c/(?P<id>\w+)/?(.*)", card_url)
        if match:
            return match.groupdict()['id']
        else:
            return quote(card_url)

    ### Users & Tokens
    def get_token_url(self, app_name, expires='30days'):
        """ Return URL for retrieving access token """
        return 'https://trello.com/1/authorize?key=%(key)s&name=%(name)s&expiration=%(expires)s&response_type=token&scope=%(scope)s' % {
            'key': self.API_KEY,
            'name': quote_plus(self.APP_NAME),
            'expires': expires,
            'scope': 'read,write'
        }


    def get_myself(self):
        return self.do_request("/tokens/%s/member" % self._access_token)

    def get_member(self, member_name):
        return self.do_request("/members/%s" % member_name)

    ### Boards
    def get_board(self, board_id):
        return self.do_request("/boards/%s" % board_id)

    ### Columns
    def get_columns(self, board_id):
        return self.do_request("/boards/%s/lists" % board_id)

    def get_column(self, column_id):
        return self.do_request("/lists/%s" % column_id)

    ### Cards
    def get_card(self, card_id=None, card_url=None):
        if card_url and not card_id:
            card_id = self.get_card_id(card_url)
        return self.do_request("/cards/%s" % card_id)

    def get_cards(self, column_id):
        return self.do_request("/lists/%s/cards" % column_id)

    def create_card(self, name, description, list_id):
        return self.do_request(
            '/cards',
            method='post',
            data={
                'name': name,
                'desc': description,
                'idList': list_id
            }
        )


    def move_card(self, card_id, board_id=None, column_id=None):
        """
        Move card to the given column on another board.

        If no column is given, it will be placed in the first one.
        If no board is given, column is assumed to be on the same boards.

        """
        if board_id:
            self.do_request("/cards/%s/idBoard" % card_id, data={'value': board_id}, method='put')

        if column_id:
            self.do_request("/cards/%s/idList" %  card_id, data={'value': column_id}, method='put')

    def comment_card(self, card_id, comment):
        self.do_request("/cards/%s/actions/comments" % card_id, method='post', data={'text': comment})

    def add_card_member(self, card_id, member_id):
        self.do_request(
            "/cards/%s/members" % card_id,
            method='post',
            data={
                'value': member_id
            }   
        )

    def label_card(self, card_id, label):
        self.do_request(
            "/cards/%s/labels" % card_id,
            method='post',
            data={
                'value': label
            }
        )


    ### Checklists
    def create_item(self, checklist_id, name, pos):
        """ Create new item in the given checklist on given position """
        return self.do_request(
            url="/checklists/%s/checkItems" % checklist_id,
            method='post',
            data={
                'name': name,
                'post': pos
            }
        )

    def check_item(self, card_id, checklist_id, item_id):
        """ Mark item in the given checklist as complete """
        # OK, WTF
        # This is kinda underdocumented, method is not present in API,
        # but inspecting the requests in live trello says yes, they do
        # https://trello.com/1/cards/5352e7118793950e77eb1c31/checklist/5352e75978962c0c7778f601/checkItem/5352fb5abb1fb4ca20b7be44
        self.do_request(
            "/cards/%(card_id)s/checklist/%(checklist_id)s/checkItem/%(item_id)s" % {
                'checklist_id': checklist_id,
                'item_id': item_id,
                'card_id': card_id
            },
            method='put',
            data={
                'state': 'complete'
            }
        )

    def get_card_checklists(self, card_id):
        return self.do_request('/cards/%s/checklists' % card_id)

    def get_checklist_items(self, checklist_id):
        return self.do_request('/checklists/%s/checkItems' % checklist_id)

    def delete_checklist_item(self, checklist_id, checklist_item_id):
        return self.do_request(
            "/checklists/%s/checkItems/%s" % (checklist_id, checklist_item_id),
            method='delete'
        )

    def add_column(self, board_id, name, position):
        """ Add position^th column to the board_id. Position is 1-indexed """
        # Position is not just an integer as in 3 for 3rd from the left,
        # but we ultimately want our API to look act that way
        # Therefore, find out position-1 & increment
        columns = self.get_columns(board_id=board_id)

        trello_position = 'bottom' # default

        if len(columns) >= position - 1 and position > 1:
            # -2: -1 for prev card, additional -1 because list is 0-indexed
            # +1 for pos as we want to have it behind it
            trello_position = columns[position - 2]['pos'] + 1

        return self.do_request(
            "/boards/%s/lists" % (board_id,),
            method='post',
            data={
                'name': name,
                'pos': trello_position
            }
        )



import click

from blackbelt.handle_github import (
    pull_request,
    merge as do_merge,
    deploy as do_deploy
)


@click.group(help='Handle github-related tasks and integrations.')
def cli():
    pass


@cli.command()
@click.argument('card_url', required=False)
def pr(card_url):
    pull_request(card_url)


@cli.command()
@click.argument('pr_url')
def merge(pr_url):
    do_merge(pr_url)


@cli.command()
@click.argument('pr_url')
def deploy(pr_url):
    do_deploy(pr_url)
